package cc.smtweb.framework.core.common;

import lombok.Data;

/**
 * Created by Akmm at 2022/3/23 9:39
 * 系统的一些枚举变量
 */
public interface SwEnum {


    /**
     * Created by Akmm at 2022/2/9 10:01
     * 字段业务类别
     */
    class FieldTypeBean extends AbstractEnum.IntEnumBean {
        public String dataType;

        public FieldTypeBean(Integer value, String name, String dataType) {
            super(value, name);
            this.dataType = dataType;
        }
    }

    class FieldType extends AbstractEnum<Integer, FieldTypeBean> {
        public static FieldType instance = new FieldType();

        public static IntEnumBean ID = instance.addEnum(1, "主键", DataType.ID.value);
        public static IntEnumBean CODE = instance.addEnum(2, "编码字段", DataType.CODE.value);
        public static IntEnumBean NAME = instance.addEnum(3, "名称字段", DataType.NAME.value);
        public static IntEnumBean PARENT_ID = instance.addEnum(4, "父ID", DataType.ID.value);
        public static IntEnumBean LEVEL_CODE = instance.addEnum(5, "级次码", DataType.CODE.value);
        public static IntEnumBean ORDER = instance.addEnum(6, "排序字段", "");
        public static IntEnumBean CREATE_USER = instance.addEnum(7, "创建人", DataType.ID.value);
        public static IntEnumBean CREATE_TIME = instance.addEnum(8, "创建时间", DataType.DATETIME.value);
        public static IntEnumBean UPDATE_USER = instance.addEnum(9, "更新人", DataType.ID.value);
        public static IntEnumBean LAST_TIME = instance.addEnum(10, "更新时间", DataType.DATETIME.value);

        @Override
        protected FieldTypeBean buildBean(Integer value, String name) {
            return new FieldTypeBean(value, name, "");
        }

        public FieldTypeBean addEnum(Integer value, String name, String dataType) {
            final FieldTypeBean bean = new FieldTypeBean(value, name, dataType);
            mapAll.put(value, bean);
            return bean;
        }
    }

    /**
     * 字段编辑类型
     */
    class EditorType extends StrEnum {
        public static EditorType instance = new EditorType();

        public static StrEnumBean INPUT = instance.addEnum("input", "文本");
        public static StrEnumBean TEXT = instance.addEnum("text", "长文本");
        public static StrEnumBean NUMBER = instance.addEnum("num", "数字");
        public static StrEnumBean DATE = instance.addEnum("date", "日期");
        public static StrEnumBean TIME = instance.addEnum("time", "时间");
        public static StrEnumBean DATETIME = instance.addEnum("datetime", "日期时间");
        public static StrEnumBean COMBO = instance.addEnum("combo", "下拉");
        public static StrEnumBean TREE = instance.addEnum("tree", "树型");

    }

    /**
     * 数据字段类型Bean
     */
    class DataTypeBean extends AbstractEnum.StrEnumBean {
        public String sqlType;
        public int dataLength;
        public String javaType;
        public String defaultValue;
        public String editor;

        public DataTypeBean(String value, String name, String sqlType, int dataLength, String javaType, String defaultValue, String editor) {
            super(value, name);
            this.sqlType = sqlType;
            this.dataLength = dataLength;
            this.javaType = javaType;
            this.defaultValue = defaultValue;
            this.editor = editor;
        }
    }

    /**
     * 数据类型定义，参见design_db.yaml配置
     */
    class DataType extends AbstractEnum<String, DataTypeBean> {
        public final static String TYPE_BOOL = "bool";
        public final static String TYPE_DATE = "date";
        public final static String TYPE_DATETIME = "datetime";

        public static DataType instance = new DataType();
        public static DataTypeBean ID = instance.addEnum("id", "ID", "bigint", 0, "long", "", EditorType.INPUT.value);
        public static DataTypeBean CODE = instance.addEnum("code", "编码", "varchar", 32, "string", "", EditorType.INPUT.value);
        public static DataTypeBean NAME = instance.addEnum("name", "名称", "varchar", 100, "string", "", EditorType.INPUT.value);
        public static DataTypeBean REMARK = instance.addEnum("remark", "备注", "varchar", 255, "string", "", EditorType.INPUT.value);
        public static DataTypeBean TEXT = instance.addEnum("text", "大文本", "text", 0, "string", "", EditorType.TEXT.value);
        public static DataTypeBean INT = instance.addEnum("int", "整型", "int", 0, "int", "0", EditorType.NUMBER.value);
        public static DataTypeBean SHORT = instance.addEnum("short", "短整型", "smallint", 0, "short", "0", EditorType.NUMBER.value);
        public static DataTypeBean BOOL = instance.addEnum("bool", "布尔型", "tinyint", 0, "boolean", "0", EditorType.COMBO.value);
        public static DataTypeBean CURRENCY = instance.addEnum("currency", "金额型", "bigint", 0, "long", "0", EditorType.NUMBER.value);
        public static DataTypeBean DATE = instance.addEnum("date", "日期型", "bigint", 0, "long", "0", EditorType.DATE.value);
        public static DataTypeBean TIME = instance.addEnum("time", "时间型", "bigint", 0, "long", "0", EditorType.TIME.value);
        public static DataTypeBean DATETIME = instance.addEnum("datetime", "时间型", "bigint", 0, "long", "0", EditorType.DATETIME.value);

        @Override
        protected DataTypeBean buildBean(String value, String name) {
            return null;
        }

        public DataTypeBean addEnum(String value, String name, String sqlType, int dataLength, String javaType, String defaultValue, String editor) {
            final DataTypeBean bean = new DataTypeBean(value, name, sqlType, dataLength, javaType, defaultValue, editor);
            mapAll.put(value, bean);
            return bean;
        }
    }

    /**
     * 表类型
     */
    class TableType extends IntEnum {
        public static TableType instance = new TableType();

        public static AbstractEnum.IntEnumBean TYPE_GENERAL = instance.addEnum(0, "普通表");
        public static AbstractEnum.IntEnumBean TYPE_TREE = instance.addEnum(1, "树型表");
        public static AbstractEnum.IntEnumBean TYPE_CODE = instance.addEnum(2, "编码表");
        public static AbstractEnum.IntEnumBean TYPE_ABSTACT = instance.addEnum(3, "虚拟抽象表");
        public static AbstractEnum.IntEnumBean TYPE_VIEW = instance.addEnum(4, "视图");

    }
}
